#include "testApp.h"

//--------------------------------------------------------------
void testApp::setup(){
	
	ofSetVerticalSync(true);
	ofSetLineWidth(6);
	
	liveVidInfo.setup();
	bInfoGrabbed=false;
	bTriggered=false;
	bAppView=false;
	
	howManyParticlesX=80;
	howManyParticlesY=60;
	
	myTexture.allocate(liveVidInfo.vidWidth, liveVidInfo.vidHeight, GL_RGB);
	
	
	for (int j=0;j<howManyParticlesY;j++){
	for (int i = 0; i < howManyParticlesX; i++){
		
			particle myParticle;
			myParticle.setInitialCondition(myParticle.particleRad*2*i+myParticle.particleRad,myParticle.particleRad*2*j+myParticle.particleRad,0,0);
		myParticle.initPos.x=myParticle.particleRad*2*i+myParticle.particleRad;
		myParticle.initPos.y=myParticle.particleRad*2*j+myParticle.particleRad;
			particles.push_back(myParticle);
		}
	}
	
	VF.setupField(60,40,ofGetWidth(), ofGetHeight());
	
	
	

	
}

//--------------------------------------------------------------
void testApp::update(){
	
	for (int i = 0; i < particles.size(); i++){
		particles[i].resetForce();
		
		// get the force from the vector field: 
		ofxVec2f frc;
		frc = VF.getForceFromPos(particles[i].pos.x, particles[i].pos.y);
		particles[i].addForce(frc.x, frc.y);
		particles[i].addDampingForce();
		particles[i].update();
		
	}


	liveVidInfo.update();
	
	if (bInfoGrabbed) {
		
		unsigned char * newPixels=liveVidInfo.vidGrabber.getPixels();
		
		myTexture.loadData(newPixels, liveVidInfo.vidWidth, liveVidInfo.vidHeight, GL_RGB);
		
		int unitCountX=(liveVidInfo.vidWidth/howManyParticlesX);
		int unitCountY=(liveVidInfo.vidHeight/howManyParticlesY);
		int counter=0;
		for (int j=0;j<liveVidInfo.vidHeight;j+=unitCountY){
			for (int i=0;i<liveVidInfo.vidWidth;i+=unitCountX){
				
				particles[counter].myColor.r=newPixels[j*3*liveVidInfo.vidWidth+i*3];
				particles[counter].myColor.g=newPixels[j*3*liveVidInfo.vidWidth+i*3+1];
				particles[counter].myColor.b=newPixels[j*3*liveVidInfo.vidWidth+i*3+2];
				counter++;
				//printf("counter: %d\n",counter);
			}
		}
		
		
		
		
		//bInfoGrabbed=false;   //------------------------------------if uncommented, makes the program work with still images
	}
	
}

//--------------------------------------------------------------
void testApp::draw(){
	
	ofBackground(0, 0, 0);
	
	if (!bAppView){
		ofSetColor(255, 255, 255);
		liveVidInfo.draw(50,50);
	if (bTriggered) {
		myTexture.draw(500,50);	
	}
	}else {

		
		for (int i = 0; i < particles.size(); i++){
			particles[i].draw();
		}
	}

	
}

//--------------------------------------------------------------
void testApp::keyPressed(int key){

}

//--------------------------------------------------------------
void testApp::keyReleased(int key){
	if (key==' ') {
		liveVidInfo.grabInfo();
		bInfoGrabbed=true;
	
	if (!bTriggered) {
		bTriggered=true;
		}
	}
	if ((key=='a')||(key=='A')) {
		bAppView=!bAppView;
	}
	if ((key=='c')||(key=='C')){
		VF.clear();
	}
	if ((key=='n')||(key=='N')){
		VF.clear();
		
			particles.clear();	
		
		for (int j=0;j<howManyParticlesY;j++){
			for (int i = 0; i < howManyParticlesX; i++){
				
				particle myParticle;
				myParticle.setInitialCondition(myParticle.particleRad*2*i+myParticle.particleRad,myParticle.particleRad*2*j+myParticle.particleRad,0,0);
				myParticle.initPos.x=myParticle.particleRad*2*i+myParticle.particleRad;
				myParticle.initPos.y=myParticle.particleRad*2*j+myParticle.particleRad;
				particles.push_back(myParticle);
			}
		}
	}

}

//--------------------------------------------------------------
void testApp::mouseMoved(int x, int y ){

}

//--------------------------------------------------------------
void testApp::mouseDragged(int x, int y, int button){
	float diffx = x - prevMouseX;
	float diffy = y - prevMouseY;
	
	VF.addVectorCircle((float)x, (float)y, diffx*0.3, diffy*0.3, 60, 0.3f);
	
	prevMouseX = x;
	prevMouseY = y;
}

//--------------------------------------------------------------
void testApp::mousePressed(int x, int y, int button){
	prevMouseX = x; 
	prevMouseY = y;
}

//--------------------------------------------------------------
void testApp::mouseReleased(int x, int y, int button){

}

//--------------------------------------------------------------
void testApp::windowResized(int w, int h){

}

